package com.persagy.dmp.dic.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.basic.utils.QueryCriteriaHelper;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.common.model.response.PageList;
import com.persagy.dmp.common.utils.ResultHelper;
import com.persagy.dmp.dic.entity.DictDataMap;
import com.persagy.dmp.dic.service.DictDataMapService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 字典数据映射
 * @author:linhuili
 * @date:2021/6/10
 */
@RestController
@RequestMapping(value = "/dic/dt/data/map")
public class DictDataMapController {

    @Autowired
    private DictDataMapService service;

    @PostMapping("/query")
    public CommonResult<PageList<DictDataMap>> query(@RequestBody QueryCriteria criteria) {
        if(criteria == null) {
            throw new BusinessException(ResponseCode.A0400.getCode(), ResponseCode.A0400.getDesc());
        }
        QueryWrapper<DictDataMap> wrapper = new QueryWrapper<>();
        // 转换查询条件
        QueryCriteriaHelper.toWrapper(wrapper, criteria);
        Page page = service.queryByCondition(QueryCriteriaHelper.toPage(criteria), wrapper);
        return ResultHelper.multi(page.getRecords(), page.getTotal());
    }

    @PostMapping("/create")
    public CommonResult<DictDataMap> create(@RequestBody DictDataMap vo){
        vo = service.insert(vo);
        return ResultHelper.single(vo);
    }

    @PostMapping("/update")
    public CommonResult<DictDataMap> update(@RequestBody DictDataMap vo){
        vo = service.update(vo);
        return ResultHelper.single(vo);
    }

    @PostMapping("/delete")
    public CommonResult<DictDataMap> delete(@RequestBody DictDataMap vo){
        service.delete(vo);
        return ResultHelper.success();
    }
}
