package com.persagy.dmp.rwd.digital.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.rwd.basic.constant.DigitalMessageConstant;
import com.persagy.dmp.rwd.basic.utils.DigitalCommonUtils;
import com.persagy.dmp.rwd.basic.utils.DigitalMessageHelper;
import com.persagy.dmp.rwd.define.service.IObjectInfoService;
import com.persagy.dmp.common.constant.ValidEnum;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.rwd.digital.dao.ObjectDigitalMapper;
import com.persagy.dmp.rwd.digital.entity.ObjectDigital;
import com.persagy.dmp.rwd.digital.service.IObjectDigitalService;
import com.persagy.dmp.rwd.parser.service.InfoDataFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * 对象数据 实现类
 * @author Charlie Yu
 * @date 2021-07-08
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ObjectDigitalServiceImpl implements IObjectDigitalService {

    @Resource
    private ObjectDigitalMapper dao;
    @Autowired
    private DigitalMessageHelper messageSender;
    @Autowired
    private IObjectInfoService infoService;

    @Override
    public ObjectDigital load(String id) {
        return dao.selectById(id);
    }

    @Override
    public List<ObjectDigital> queryByCondition(Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectList(queryWrapper);
    }

    @Override
    public Page queryByCondition(Page page, Wrapper<ObjectDigital> queryWrapper) {
        return dao.selectPage(page, queryWrapper);
    }

    @Override
    public List<ObjectDigital> insert(List<ObjectDigital> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        validateOnSave(voList, false);
        voList.forEach(vo -> {
            // 生成ID
            vo.setId(DigitalCommonUtils.fetchObjectId(vo.getProjectId(), vo.getObjType()));
            // 新增
            dao.insert(vo);
            // 新增后消息
            messageSender.sendMessage(DigitalMessageConstant.OPERATE_AFTER_INSERT, null, vo, false);
        });
        return voList;
    }


    @Override
    public List<ObjectDigital> update(List<ObjectDigital> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        validateOnSave(voList, true);
        List<String> idList = CollUtil.getFieldValues(voList, BaseEntity.PROP_ID, String.class);
        List<ObjectDigital> dbList = dao.selectBatchIds(idList);
        Map<String, ObjectDigital> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        voList.forEach(vo -> {
            ObjectDigital dbVO = MapUtil.get(dbMap, vo.getId(), ObjectDigital.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 修改
            dao.updateById(vo);
            // 修改后消息
            messageSender.sendMessage(DigitalMessageConstant.OPERATE_AFTER_UPDATE, dbVO, vo, false);

        });
        return voList;
    }

    /**
     * 保存校验
     * 编码全局唯一
     * @param voList
     * @param isUpdate
     */
    private void validateOnSave(List<ObjectDigital> voList, boolean isUpdate) {
        validateInfoCode(voList);
    }

    /**
     * 信息点值校验
     * @param voList
     */
    private void validateInfoCode(List<ObjectDigital> voList) {
        // 查询信息点定义
        List<String> typeCodeList = CollUtil.getFieldValues(voList, "classCode", String.class);
        Map<String, Map<String, ObjectInfoDefine>> allInfoMap = infoService.queryByTypes(typeCodeList);
        // 信息点校验器
        for(ObjectDigital vo:voList) {
            Map<String, ObjectInfoDefine> infoMap = allInfoMap.get(vo.getClassCode());
            if(MapUtil.isEmpty(infoMap)) {
                continue;
            }
            // 信息点值对象
            ObjectNode infos = vo.getInfos();
            if(infos == null || infos.isNull()) {
                continue;
            }
            // 按infos属性循环处理
            Iterator<String> infoKeys = infos.fieldNames();
            while(infoKeys.hasNext()) {
                String infoCode = infoKeys.next();
                // 如果信息点中不存在，移除
                if(!infoMap.containsKey(infoCode)) {
                   // infos.remove(infoCode);
                    infoKeys.remove();
                    continue;
                }
                // 校验信息点值
                InfoDataFactory.parse(infoMap.get(infoCode), infos.get(infoCode));

            }
        }
    }

    @Override
    public void delete(List<String> idList) {
        if(CollUtil.isEmpty(idList)) {
            return;
        }
        List<ObjectDigital> dbList = dao.selectBatchIds(idList);
        Map<String, ObjectDigital> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        idList.forEach(id -> {
            ObjectDigital dbVO = MapUtil.get(dbMap, id, ObjectDigital.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 删除
            ObjectDigital vo = new ObjectDigital();
            vo.setId(id);
            vo.setValid(ValidEnum.FALSE.getType());
            dao.updateById(vo);
            // 删除后消息
            dbVO.setValid(ValidEnum.FALSE.getType());
            messageSender.sendMessage(DigitalMessageConstant.OPERATE_AFTER_DELETE, null, dbVO, false);
        });
    }
}
