package com.persagy.dmp.basic.utils;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.JsonNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.model.entity.BaseEntity;

import java.util.Iterator;

/**
 * 查询条件 助手
 * @author Charlie Yu
 * @date 2021-06-25
 */
public class QueryCriteriaHelper {

    /**
     * 转换为Page分页信息
     * @param criteria
     * @return
     */
    public static Page toPage(QueryCriteria criteria) {
        if(criteria == null) {
            criteria = new QueryCriteria();
        }
        // 当前页默认为1
        if(criteria.getPage() == null || criteria.getPage() < 1) {
            criteria.setPage(1L);
        }
        // 没有每页行数表示不分页
        if(criteria.getSize() == null || criteria.getSize() < 1) {
            criteria.setSize(-1L);
        }
        Page pageInfo = new Page();
        pageInfo.setCurrent(criteria.getPage());
        pageInfo.setSize(criteria.getSize());
        pageInfo.setOrders(criteria.getOrders());
        pageInfo.setSearchCount(!criteria.isWithoutCount());
        return pageInfo;
    }

    /**
     * 转换为查询条件
     * @param wrapper
     */
    public static <T extends BaseEntity> void toWrapper(QueryWrapper<T> wrapper, QueryCriteria criteria) {
        if(criteria == null || criteria.getCriteria() == null || criteria.getCriteria().isEmpty(null)) {
            return;
        }
        Iterator<String> fields = criteria.getCriteria().fieldNames();
        // 按字段循环
        while(fields.hasNext()) {
            String field = fields.next();
            JsonNode fieldValue = criteria.getCriteria().get(field);
            String dbField = StrUtil.toUnderlineCase(field);
            // 值节点
            if(fieldValue.isValueNode()) {
                wrapper.eq(dbField, JsonNodeUtils.getNodeValue(fieldValue));
                continue;
            }
            // 数组
            if(fieldValue.isArray()) {
                wrapper.in(dbField, JsonNodeUtils.getListValue(fieldValue));
                continue;
            }
            // 对象
            if(fieldValue.isObject()) {
                Iterator<String> valueOperators = fieldValue.fieldNames();
                while(valueOperators.hasNext()) {
                    String valueOperator = valueOperators.next();
                    JsonNode operatorValue = fieldValue.get(valueOperator);
                    addCondition(wrapper, dbField, valueOperator,operatorValue);
                }
            }
        }
    }

    /**
     * 按条件类型添加查询条件
     * @param wrapper
     * @param field
     * @param operator
     * @param valueNode
     */
    private static <T extends BaseEntity> void addCondition(QueryWrapper<T> wrapper, String field, String operator, JsonNode valueNode) {
        if("$ne".equalsIgnoreCase(operator)) {
            wrapper.ne(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$gt".equalsIgnoreCase(operator)) {
            wrapper.gt(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$gte".equalsIgnoreCase(operator)) {
            wrapper.ge(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$lt".equalsIgnoreCase(operator)) {
            wrapper.lt(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$lte".equalsIgnoreCase(operator)) {
            wrapper.le(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$in".equalsIgnoreCase(operator)) {
            wrapper.in(field, JsonNodeUtils.getListValue(valueNode));
        } else if("$like".equalsIgnoreCase(operator)) {
            wrapper.like(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$notLike".equalsIgnoreCase(operator)) {
            wrapper.notLike(field, JsonNodeUtils.getNodeValue(valueNode));
        } else if("$null".equalsIgnoreCase(operator)) {
            boolean nullFlag = valueNode.booleanValue();
            if(nullFlag) {
                wrapper.isNull(field);
            } else {
                wrapper.isNotNull(field);
            }
        }
    }
}
